#include<string.h>
#include<fstream.h>
#include<list.h>
#include<gtk--.h>
#include"Movable.h"
#include"Drawable.h"
#include"Selectable.h"
#include"Shape.h"
#include"Line.h"
#include"Rectangle.h"
#include"Arc.h"
#include"Text.h"
#include"OhharaCanvas.h"
#include"OhharaDraw.h"

OhharaDraw::OhharaDraw() :
	fileItem("File"),newItem("New"),openItem("Open"),saveItem("Save"),
	quitItem("Quit"),helpItem("Help"),aboutItem("Help"),
	newPixmap(this,"pixmap/new.xpm"),
	openPixmap(this,"pixmap/open.xpm"),
	savePixmap(this,"pixmap/save.xpm"),
	helpPixmap(this,"pixmap/help.xpm"),
	linePixmap(this,"pixmap/line.xpm"),
	rectanglePixmap(this,"pixmap/rectangle.xpm"),
	arcPixmap(this,"pixmap/arc.xpm"),
	textPixmap(this,"pixmap/text.xpm"),
	filledPixmap(this,"pixmap/filled.xpm"),
	regularPixmap(this,"pixmap/regular.xpm"),
	colorSelectionPixmap(this,"pixmap/colorsel.xpm"),
	movePixmap(this,"pixmap/move.xpm"),
	removePixmap(this,"pixmap/remove.xpm"),
	ohharaCanvasTable(2,2,false),
	state(line),
	colorSelectionDialog("Select Color"),
	openFileSelection("Open File"),
	saveFileSelection("Save File"),
	ohharaDrawPixmap(this,"pixmap/ohharadraw.xpm"),
	ohharaDrawLabel("Programmed by ohhara@postech.ac.kr (1998/05/28)"),
	ohharaDrawLicense("This application can be distributed freely"),
	ohharaDrawButton("OK")
{

	gdk_color_parse("black",&foregroundColor);
	gdk_color_parse("white",&backgroundColor);

	add(&ohharaDrawVBox);
	ohharaDrawVBox.show();


	// Menu
	fileMenu.append(&newItem);
	fileMenu.append(&openItem);
	fileMenu.append(&saveItem);
	fileMenu.append(&quitItem);
	helpMenu.append(&aboutItem);

	fileItem.show();
	newItem.show();
	openItem.show();
	saveItem.show();
	quitItem.show();

	helpItem.show();
	aboutItem.show();

	fileItem.set_submenu(&fileMenu);
	helpItem.set_submenu(&helpMenu);

	ohharaDrawMenuBar.append(&fileItem);
	ohharaDrawMenuBar.append(&helpItem);

	ohharaDrawVBox.pack_start(&ohharaDrawMenuBar,false,false);
	ohharaDrawMenuBar.show();


	// Menu signal
	connect_to_method(newItem.activate,this,
		&newButton_clicked_impl);
	connect_to_method(openItem.activate,this,
		&openButton_clicked_impl);
	connect_to_method(saveItem.activate,this,
		&saveButton_clicked_impl);
	connect_to_method(quitItem.activate,Gtk_Main::instance(),&Gtk_Main::quit);
	connect_to_method(aboutItem.activate,this,
		&aboutButton_clicked_impl);


	// Toolbar
	newButton.show();
	newPixmap.show();
	openButton.show();
	openPixmap.show();
	saveButton.show();
	savePixmap.show();
	helpButton.show();
	helpPixmap.show();

	ohharaToolbar.append_widget(&newButton,"New File");
	ohharaToolbar.append_widget(&openButton,"Open File");
	ohharaToolbar.append_widget(&saveButton,"Save File");
	ohharaToolbar.append_space();
	ohharaToolbar.append_widget(&helpButton,"Help");

	newButton.add(&newPixmap);
	openButton.add(&openPixmap);
	saveButton.add(&savePixmap);
	helpButton.add(&helpPixmap);

	ohharaDrawVBox.pack_start(&ohharaToolbar,false,false,2);
	ohharaToolbar.show();


	lineButton.show();
	linePixmap.show();
	rectangleButton.show();
	rectanglePixmap.show();
	arcButton.show();
	arcPixmap.show();
	textButton.show();
	textPixmap.show();
	filledToggleButton.show();
	filledPixmap.show();
	regularToggleButton.show();
	regularPixmap.show();
	colorSelectionButton.show();
	colorSelectionPixmap.show();
	moveButton.show();
	movePixmap.show();
	removeButton.show();
	removePixmap.show();

	ohharaDrawToolbar.append_widget(&lineButton,"Draw Line");
	ohharaDrawToolbar.append_widget(&rectangleButton,"Draw Rectangle");
	ohharaDrawToolbar.append_widget(&arcButton,"Draw Arc");
	ohharaDrawToolbar.append_widget(&textButton,"Input Text");
	ohharaDrawToolbar.append_space();
	ohharaDrawToolbar.append_widget(&filledToggleButton,"Filled Object");
	ohharaDrawToolbar.append_widget(&regularToggleButton,"Regular Object");
	ohharaDrawToolbar.append_space();
	ohharaDrawToolbar.append_widget(&colorSelectionButton,"Select Color");
	ohharaDrawToolbar.append_widget(&moveButton,"Move Object");
	ohharaDrawToolbar.append_widget(&removeButton,"Remove Object");

	lineButton.add(&linePixmap);
	rectangleButton.add(&rectanglePixmap);
	arcButton.add(&arcPixmap);
	textButton.add(&textPixmap);
	filledToggleButton.add(&filledPixmap);
	regularToggleButton.add(&regularPixmap);
	colorSelectionButton.add(&colorSelectionPixmap);
	moveButton.add(&movePixmap);
	removeButton.add(&removePixmap);

	ohharaDrawVBox.pack_start(&ohharaDrawToolbar,false,false,2);
	ohharaDrawToolbar.show();


	// Toolbar signal
	connect_to_method(newButton.clicked,this,
		&newButton_clicked_impl);
	connect_to_method(openButton.clicked,this,
		&openButton_clicked_impl);
	connect_to_method(saveButton.clicked,this,
		&saveButton_clicked_impl);
	connect_to_method(helpButton.clicked,this,
		&aboutButton_clicked_impl);

	connect_to_method(lineButton.clicked,this,
		&lineButton_clicked_impl);
	connect_to_method(rectangleButton.clicked,this,
		&rectangleButton_clicked_impl);
	connect_to_method(arcButton.clicked,this,
		&arcButton_clicked_impl);
	connect_to_method(textButton.clicked,this,
		&textButton_clicked_impl);
	connect_to_method(colorSelectionButton.clicked,this,
		&colorSelectionButton_clicked_impl);
	connect_to_method(moveButton.clicked,this,
		&moveButton_clicked_impl);
	connect_to_method(removeButton.clicked,this,
		&removeButton_clicked_impl);


	// Canvas
	ohharaCanvas.set_usize(canvasWidth,canvasHeight);
	ohharaCanvas.show();
	ohharaCanvasTable.attach(&ohharaCanvas,1,2,1,2,GTK_FILL,GTK_FILL);
	// HRuler
	ohharaDrawHRuler.set_metric(GTK_PIXELS);
	ohharaDrawHRuler.set_range(0,canvasWidth,0,canvasWidth);
	ohharaDrawHRuler.show();
	ohharaCanvasTable.attach(&ohharaDrawHRuler,1,2,0,1,GTK_FILL,GTK_FILL);
	// VRuler
	ohharaDrawVRuler.set_metric(GTK_PIXELS);
	ohharaDrawVRuler.set_range(0,canvasHeight,0,canvasHeight);
	ohharaDrawVRuler.show();
	ohharaCanvasTable.attach(&ohharaDrawVRuler,0,1,1,2,GTK_FILL,GTK_FILL);

	ohharaCanvasTable.show();

	ohharaDrawVBox.add(&ohharaCanvasTable);

	// Canvas signal
	connect_to_method(ohharaCanvas.motion_notify_event,this,
		&emit_to_OhharaDrawHRuler);
	connect_to_method(ohharaCanvas.motion_notify_event,this,
		&ohharaCanvas_motion_notify_event_impl);
	connect_to_method(ohharaCanvas.button_press_event,this,
		&ohharaCanvas_button_press_event_impl);


	// ColorSelectionDialog signal
	connect_to_method(colorSelectionDialog.ok_button.clicked,this,
		&change_foregroundColor);
	connect_to_method(colorSelectionDialog.cancel_button.clicked,this,
		&donot_change_foregroundColor);

	// FileSelection signal
	connect_to_method(openFileSelection.ok_button.clicked,this,
		&open_file);
	connect_to_method(openFileSelection.cancel_button.clicked,this,
		&donot_open_file);
	connect_to_method(saveFileSelection.ok_button.clicked,this,
		&save_file);
	connect_to_method(saveFileSelection.cancel_button.clicked,this,
		&donot_save_file);

	// Help Dialog
	ohharaDrawPixmap.show();
	ohharaDrawLabel.show();
	ohharaDrawLicense.show();
	ohharaDrawButton.show();
	ohharaDrawHelpDialog.vbox()->pack_start(&ohharaDrawPixmap);
	ohharaDrawHelpDialog.vbox()->pack_start(&ohharaDrawLabel);
	ohharaDrawHelpDialog.vbox()->pack_start(&ohharaDrawLicense);
	ohharaDrawHelpDialog.vbox()->pack_start(&ohharaDrawButton);
	ohharaDrawHelpDialog.set_title("Ohhara Draw 0.1 Help Dialog");

	// Help Dialog signal
	connect_to_method(ohharaDrawButton.clicked,this,&close_Help);

	set_title("Ohhara Draw 0.1");
}
OhharaDraw::~OhharaDraw()
{
}
gint OhharaDraw::delete_event_impl(GdkEventAny* gdkEventAny)
{
	Gtk_Main::instance()->quit(); 
	return true;
} 
gint OhharaDraw::emit_to_OhharaDrawHRuler(GdkEventMotion* p1)
{
	ohharaDrawHRuler.motion_notify_event_callback(
		GTK_WIDGET(ohharaDrawHRuler.gtkobject),p1);
	ohharaDrawVRuler.motion_notify_event_callback(
		GTK_WIDGET(ohharaDrawVRuler.gtkobject),p1);
}
gint OhharaDraw::ohharaCanvas_motion_notify_event_impl(GdkEventMotion* e)
{
	movingX=(gint)(e->x);
	movingY=(gint)(e->y);
	x2=movingX;
	y2=movingY;
	width=x2-x1;
	height=y2-y1;

	switch(state)
	{
		case lineFirstPointSet:
			ohharaCanvas.drawing_line(x1,y1,x2,y2,foregroundColor,
				backgroundColor);
			break;
		case rectangleFirstPointSet:
			tmpX1=x;
			tmpY1=y;
			if(width<0)
			{
				tmpX1=x+width;
				width=-width;
			}
			if(height<0)
			{
				tmpY1=y+height;
				height=-height;
			}
			if(regularToggleButton.get_state())
			{
				if(width<height)
					height=width;
				else
					width=height;
			}
			ohharaCanvas.drawing_rectangle(tmpX1,tmpY1,width,height,
				foregroundColor,backgroundColor,filledToggleButton.get_state());
			break;
		case arcFirstPointSet:
			tmpX1=x;
			tmpY1=y;
			if(width<0)
			{
				tmpX1=x+width;
				width=-width;
			}
			if(height<0)
			{
				tmpY1=y+height;
				height=-height;
			}
			if(regularToggleButton.get_state())
			{
				if(width<height)
					height=width;
				else
					width=height;
			}
			ohharaCanvas.drawing_arc(tmpX1,tmpY1,width,height,
				foregroundColor,backgroundColor,filledToggleButton.get_state());
			break;
		case moveShapeSet:
			movingShape->set_pos(movingX-tmpX,movingY-tmpY);
			movingShape->set_pos1(movingX-tmpX1,movingY-tmpY1);
			movingShape->set_pos2(movingX-tmpX1+tmpX2,movingY-tmpY1+tmpY2);
			ohharaCanvas.refresh_all();
			break;
	}

	return true;
}
gint OhharaDraw::ohharaCanvas_button_press_event_impl(GdkEventButton* e)
{
	switch(e->button)
	{
		case 1:
			switch(state)
			{
				case line:
					x1=x=movingX;
					y1=y=movingY;
					state=lineFirstPointSet;
					break;
				case lineFirstPointSet:
					ohharaCanvas.draw_line(x1,y1,x2,y2,foregroundColor,
						backgroundColor);
					state=line;
					break;
				case rectangle:
					x1=x=movingX;
					y1=y=movingY;
					tmpX1=x;
					tmpY1=y;
					state=rectangleFirstPointSet;
					break;
				case rectangleFirstPointSet:
					if(width<0)
					{
						tmpX1=x+width;
						width=-width;
					}
					if(height<0)
					{
						tmpY1=y+height;
						height=-height;
					}
					ohharaCanvas.draw_rectangle(tmpX1,tmpY1,width,height,
						foregroundColor,backgroundColor,
							filledToggleButton.get_state());
					state=rectangle;
					break;
				case arc:
					x1=x=movingX;
					y1=y=movingY;
					tmpX1=x;
					tmpY1=y;
					state=arcFirstPointSet;
					break;
				case arcFirstPointSet:
					if(width<0)
					{
						tmpX1=x+width;
						width=-width;
					}
					if(height<0)
					{
						tmpY1=y+height;
						height=-height;
					}
					ohharaCanvas.draw_arc(tmpX1,tmpY1,width,height,
						foregroundColor,backgroundColor,
						filledToggleButton.get_state());
					state=arc;
					break;
				case text:
					tmpX1=movingX;
					tmpY1=movingY;
					ohharaCanvas.drawing_text(tmpX1,tmpY1,textData,foregroundColor,
						backgroundColor);
					state=textPointSet;
					break;
				case textPointSet:
					ohharaCanvas.draw_text(tmpX1,tmpY1,textData,foregroundColor,
						backgroundColor);
					textData[0]='\0';
					state=text;
					break;
				case move:
					movingShape=ohharaCanvas.find_shape(movingX,movingY);
					if(movingShape!=NULL)
					{
						tmpX=movingX-movingShape->get_x();
						tmpY=movingY-movingShape->get_y();
						tmpX1=movingX-movingShape->get_x1();
						tmpY1=movingY-movingShape->get_y1();
						tmpX2=movingShape->get_x2()-movingShape->get_x1();
						tmpY2=movingShape->get_y2()-movingShape->get_y1();
						state=moveShapeSet;
					}
					break;
				case moveShapeSet:
					state=move;
					break;
				case remove:
					ohharaCanvas.remove_shape(movingX,movingY);
					break;
			}
			break;
		case 2:
			break;
		case 3:
			switch(state)
			{
				case lineFirstPointSet:
					ohharaCanvas.delete_shape();
					state=line;
					break;
				case rectangleFirstPointSet:
					ohharaCanvas.delete_shape();
					state=rectangle;
					break;
				case arcFirstPointSet:
					ohharaCanvas.delete_shape();
					state=arc;
					break;
				case textPointSet:
					ohharaCanvas.delete_shape();
					textData[0]='\0';
					state=text;
					break;
			}
			break;
	}

	return true;
}
gint OhharaDraw::key_press_event_impl(GdkEventKey* p1)
{
	if(state==textPointSet)
	{
		if(p1->keyval==65293)
		{
			ohharaCanvas.draw_text(tmpX1,tmpY1,textData,foregroundColor,
				backgroundColor);
			textData[0]='\0';
			state=text;
			return true;
		}
		strcat(textData,p1->string);
		ohharaCanvas.drawing_text(tmpX1,tmpY1,textData,foregroundColor,
			backgroundColor);
	}

	return true;
}

void OhharaDraw::newButton_clicked_impl()
{
	ohharaCanvas.remove_all();
}
void OhharaDraw::openButton_clicked_impl()
{
	openFileSelection.show();
}
void OhharaDraw::open_file()
{
	ifstream fin(openFileSelection.get_filename());

	if(!fin)
		return;
	
	ohharaCanvas.remove_all();
	
	int shapeListSize;
	fin.read(&shapeListSize,sizeof(int));

	for(int i=0;i<shapeListSize;i++)
	{
		gint shapeType;
		fin.read(&shapeType,sizeof(gint));

		gint x,y,width,height,x1,y1,x2,y2;
		GdkColor foregroundColor;
		GdkColor backgroundColor;
		bool filled;
		gchar openText[1024];

		fin.read(&x,sizeof(gint));
		fin.read(&y,sizeof(gint));
		fin.read(&width,sizeof(gint));
		fin.read(&height,sizeof(gint));
		fin.read(&x1,sizeof(gint));
		fin.read(&y1,sizeof(gint));
		fin.read(&x2,sizeof(gint));
		fin.read(&y2,sizeof(gint));
		fin.read(&foregroundColor,sizeof(GdkColor));
		fin.read(&backgroundColor,sizeof(GdkColor));
		fin.read(&filled,sizeof(bool));
		fin.read(openText,sizeof(gchar)*1024);


		switch(shapeType)
		{
			case Shape::lineShape:
				ohharaCanvas.get_shapeList().push_back(new Line(x1,y1,x2,y2,
					foregroundColor,backgroundColor));
				break;
			case Shape::rectangleShape:
				ohharaCanvas.get_shapeList().push_back(new Rectangle(x,y,width,
					height,foregroundColor,backgroundColor,filled));
				break;
			case Shape::arcShape:
				ohharaCanvas.get_shapeList().push_back(new Arc(x,y,width,height,
					foregroundColor,backgroundColor,filled));
				break;
			case Shape::textShape:
				ohharaCanvas.get_shapeList().push_back(new Text(x,y,
					ohharaCanvas.get_textFont(),openText,foregroundColor,
					backgroundColor));
				break;
		}
	}

	ohharaCanvas.refresh_all();

	openFileSelection.hide();
}
void OhharaDraw::donot_open_file()
{
	openFileSelection.hide();
}
void OhharaDraw::saveButton_clicked_impl()
{
	saveFileSelection.show();
}
void OhharaDraw::save_file()
{
	ofstream fout(saveFileSelection.get_filename());

	if(!fout)
		return;

	list<Shape*> shapeList;
	list<Shape*>::const_iterator shapeListIterator;

	shapeList=ohharaCanvas.get_shapeList();
	shapeListIterator=shapeList.begin();

	gint shapeListSize=shapeList.size();
	fout.write(&shapeListSize,sizeof(int));

	for(gint i=0;i<shapeListSize;i++,shapeListIterator++)
	{
		gint shapeType;
		gint x,y,width,height,x1,y1,x2,y2;
		GdkColor foregroundColor;
		GdkColor backgroundColor;
		bool filled;
		gchar* saveText;

		shapeType=(*shapeListIterator)->get_shapeType();
		x=(*shapeListIterator)->get_x();
		y=(*shapeListIterator)->get_y();
		width=(*shapeListIterator)->get_width();
		height=(*shapeListIterator)->get_height();
		x1=(*shapeListIterator)->get_x1();
		y1=(*shapeListIterator)->get_y1();
		x2=(*shapeListIterator)->get_x2();
		y2=(*shapeListIterator)->get_y2();
		foregroundColor=(*shapeListIterator)->get_foregroundColor();
		backgroundColor=(*shapeListIterator)->get_backgroundColor();
		filled=(*shapeListIterator)->get_filled();
		saveText=(*shapeListIterator)->get_text();

		fout.write(&shapeType,sizeof(gint));
		fout.write(&x,sizeof(gint));
		fout.write(&y,sizeof(gint));
		fout.write(&width,sizeof(gint));
		fout.write(&height,sizeof(gint));
		fout.write(&x1,sizeof(gint));
		fout.write(&y1,sizeof(gint));
		fout.write(&x2,sizeof(gint));
		fout.write(&y2,sizeof(gint));
		fout.write(&foregroundColor,sizeof(GdkColor));
		fout.write(&backgroundColor,sizeof(GdkColor));
		fout.write(&filled,sizeof(bool));
		fout.write(saveText,sizeof(gchar)*1024);
	}

	saveFileSelection.hide();
}
void OhharaDraw::donot_save_file()
{
	saveFileSelection.hide();
}
void OhharaDraw::aboutButton_clicked_impl()
{
	ohharaDrawHelpDialog.show();
}
void OhharaDraw::close_Help()
{
	ohharaDrawHelpDialog.hide();
}
void OhharaDraw::lineButton_clicked_impl()
{
	check_state();
	state=line;
}
void OhharaDraw::rectangleButton_clicked_impl()
{
	check_state();
	state=rectangle;
}
void OhharaDraw::arcButton_clicked_impl()
{
	check_state();
	state=arc;
}
void OhharaDraw::textButton_clicked_impl()
{
	check_state();
	state=text;
}
void OhharaDraw::check_state()
{
	if(state==lineFirstPointSet||state==rectangleFirstPointSet||
		state==arcFirstPointSet||state==textPointSet||state==changeColor)
	{
		ohharaCanvas.delete_shape();
		textData[0]='\0';
	}
}
void OhharaDraw::colorSelectionButton_clicked_impl()
{
	colorSelectionDialog.show();
}
void OhharaDraw::change_foregroundColor()
{
	state=changeColor;
	check_state();
	gdouble color[3];
	colorSelectionDialog.colorsel.get_color(color);

	foregroundColor.red=(guint16)(color[0]*65535.0);
	foregroundColor.green=(guint16)(color[1]*65535.0);
	foregroundColor.blue=(guint16)(color[2]*65535.0);

	colorSelectionDialog.hide();
}
void OhharaDraw::donot_change_foregroundColor()
{
	colorSelectionDialog.hide();
}
void OhharaDraw::moveButton_clicked_impl()
{
	check_state();
	state=move;
}
void OhharaDraw::removeButton_clicked_impl()
{
	check_state();
	state=remove;
}
